# Bridge <sup><img src="https://static.npmjs.com/255a118f56f5346b97e56325a1217a16.svg" alt="ts" width="20"/></sup>

[npm-badge]: https://img.shields.io/npm/v/twa-bridge?logo=npm

[npm-link]: https://npmjs.com/package/twa-bridge

[size-badge]: https://img.shields.io/bundlephobia/minzip/twa-bridge

[license-badge]: https://img.shields.io/github/license/telegram-web-apps/bridge

[tree-shaking-badge]: https://img.shields.io/badge/Tree%20Shaking-enabled-success

[tree-shaking-link]: https://webpack.js.org/guides/tree-shaking/

[gh-org-badge]: https://img.shields.io/badge/-Ecosystem_Component-%23555?logo=github

[gh-org-link]: https://github.com/Telegram-Web-Apps

[![Ecosystem Component][gh-org-badge]][gh-org-link]
[![Tree Shaking][tree-shaking-badge]][tree-shaking-link]
[![NPM][npm-badge]][npm-link]
![Size][size-badge]
![License][license-badge]

Communication layer between native and client applications.

## Installation

```bash  
npm i twa-bridge
```

or

```bash  
yarn add twa-bridge
```

## Usage

### Initializing

`Bridge` component has rather simple API. Its main purpose is to provide
communication channel between native and client applications, so you could
call native methods and receive events from native application.

To start using bridge, it is enough just to initialize its instance:

```typescript
import {Bridge} from 'twa-bridge';

const bridge = new Bridge();
```

Bridge constructor will automatically run required lifecycle scripts
and breathe life into the current instance. To be more accurate, constructor will
append special handlers to global window object, which are used by
Telegram native applications.

### Posting events

To call Web Apps methods, it is enough to call `Bridge`s `postEvent` method.
Let's take a look on simple example:

```typescript
import {Bridge} from 'twa-bridge';

const bridge = new Bridge();

// Close current application.
bridge.postEvent('web_app_close')
```

Some events don't require parameters, and they can be called without additional
second parameter. But there are some events, which do require:

```typescript
import {Bridge} from 'twa-bridge';

const bridge = new Bridge();

// Open new popup.
bridge.postEvent('web_app_open_popup', {
  title: 'Web Apps Notification',
  message: 'I love Web Apps',
  buttons: [{type: 'ok', id: 'ok'}]
});
```

#### Awaitable events

We should pay attention to fact, that some events are awaitable. It means, their
posting will return Promise with some result, which depends on called method.

Current list of awaitable events:
`web_app_request_viewport`, `web_app_open_popup`, `web_app_request_theme`

To learn more about which parameters and return type events have, dive into
[this](./src/events) folder.

Here is example of opening popup and getting clicked button:

```typescript
import {Bridge} from 'twa-bridge';

(async () => {
  const bridge = new Bridge();

  // Let's open new popup.
  const {button_id: btnId} = await bridge.postEvent('web_app_open_popup', {
    title: 'Web Apps Notification',
    message: 'I love Web Apps',
    buttons: [{type: 'ok', id: 'ok'}]
  });

  if (btnId === null) {
    console.log('User pressed popup cross or clicked outside.');
  } else {
    console.log(`User pressed button with id "${btnId}"`);
  }
})();
```

### Tracking events

Tracking of events is rather simple thing too. For this purpose, we could
use such methods as `on` and `subscribe`. To remove event listeners, we
use `off` and `unsubscribe`:

```typescript
import {Bridge, BridgeEventListener} from 'twa-bridge';
import {GlobalListener} from 'twa-core';

const bridge = new Bridge();
const listener: BridgeEventListener<'viewport_changed'> = payload => {
  console.log('Viewport changed:', payload)
};
const globalListener: GlobalListener = (event, ...args) => {
  console.log('Event was called:', event, ...args);
};

// Add event listener.
bridge.on('viewport_changed', listener);

// Add any event listener.
bridge.subscribe(globalListener);

// Remove event listeners.
bridge.off('viewport_changed', listener);
bridge.unsubscribe(globalListener);
```

### Debugging

`Bridge` supports debug mode which outputs additional log messages into console.
To enable this mode, it is required to pass `debug: true` while creating
new `Bridge` instance.

```typescript
import {Bridge} from 'twa-bridge';

const bridge = new Bridge({debug: true});
```

Or, in case when instance already exists, just change
its `debug` property to `true`.

```typescript
import {Bridge} from 'twa-bridge';

const bridge = new Bridge();
bridge.debug = true;
```

As a result, you will see a bunch of logs in console with information about
current bridge state.

## Higher-level control

As long as bridge provides only low-level control, we recommend using 
[SDK](https://github.com/Telegram-Web-Apps/client-sdk), which implements 
bridge methods. It also provides additional parameters checks, more intuitive 
method names and easier usage.

## Contribution

Any contribution is appreciated. Feel free to create new feature requests, bug
reports etc.
